/*
 * Copyright (c) 2005-2013, BearWare.dk
 * 
 * Contact Information:
 *
 * Bjoern D. Rasmussen
 * Bedelundvej 79
 * DK-9830 Taars
 * Denmark
 * Email: contact@bearware.dk
 * Phone: +45 20 20 54 59
 * Web: http://www.bearware.dk
 *
 * This source code is part of the TeamTalk 4 SDK owned by
 * BearWare.dk. All copyright statements may not be removed 
 * or altered from any source distribution. If you use this
 * software in a product, an acknowledgment in the product 
 * documentation is required.
 *
 */

#include "userdesktopdlg.h"
#include "mainwindow.h"

extern TTInstance* ttInst;

UserDesktopDlg::UserDesktopDlg(int userid, const QSize& size, 
                               QWidget * parent/* = 0*/)
: QDialog(parent, Qt::Dialog | Qt::WindowMinMaxButtonsHint | Qt::WindowSystemMenuHint)
, m_autosize(false)
{
    ui.setupUi(this);
    setWindowIcon(QIcon(APPICON));

    this->resize(size);
    ui.userdesktopWidget->m_userid = userid;
    connect(this, SIGNAL(userDesktopWindowUpdate(int,int)),
            ui.userdesktopWidget, SLOT(slotDesktopUpdate(int,int)));
    connect(ui.userdesktopWidget, SIGNAL(userDesktopWindowEnded(int)),
            SIGNAL(userDesktopWindowEnded(int)));
    connect(this, SIGNAL(userDesktopCursorUpdate(int,int)),
            ui.userdesktopWidget, SLOT(slotDesktopCursorUpdate(int,int)));
    ui.userdesktopWidget->slotDesktopUpdate(userid, -1);
    slotUserUpdate(userid, 0);

    initBackground();
}

UserDesktopDlg::~UserDesktopDlg()
{
    emit(userDesktopDlgClosing(ui.userdesktopWidget->m_userid));
}

void UserDesktopDlg::initBackground()
{
    //set black as background color
    ui.userdesktopWidget->setAutoFillBackground(true);
    QPalette p = ui.userdesktopWidget->palette();
    p.setBrush(QPalette::Background, QBrush(Qt::black));
    ui.userdesktopWidget->setPalette(p);
}

void UserDesktopDlg::slotUserUpdate(int userid, int /*channelid*/)
{
    if(userid != ui.userdesktopWidget->m_userid)
        return;

    if(userid == 0)
        setWindowTitle(tr("Desktop") + QString(" - ") + tr("Myself"));
    else
    {
        User user;
        if(TT_GetUser(ttInst, userid, &user))
            setWindowTitle(tr("Desktop") + QString(" - ") + _Q(user.szNickname));
    }
}
