using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using BearWare;

namespace TeamTalk.NET
{
    public partial class VideoDlg : Form
    {
        TeamTalk4 ttclient;
        int userid;
        Bitmap bmp;
        int unsubscribe_cmdid = 0;
        int stream_id = 0;

        public VideoDlg(TeamTalk4 tt, int userid)
        {
            ttclient = tt;
            this.userid = userid;
            InitializeComponent();
            this.CenterToScreen();

            User user;
            if (userid > 0 && ttclient.GetUser(userid, out user))
                this.Text = "Video - " + user.szNickname;
            else
                this.Text = "Local Video";

            ttclient.OnUserVideoFrame += new TeamTalk4.UserVideoFrame(ttclient_OnUserVideoFrame);
        }

        private void VideoDlg_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (this.userid > 0 && unsubscribe_cmdid == 0)
            {
                if (MessageBox.Show("Do you wish to stop receiving video from this user?", "Close Video",
                    MessageBoxButtons.YesNo) == DialogResult.Yes)
                {
                    //we wait for the server to repond to us that we'll no 
                    //longer receive video from this user.
                    ttclient.OnCmdProcessing += new TeamTalk4.CommandProcessing(ttclient_OnCmdProcessing);

                    //figure out how we're subscribing to video
                    Subscription subtype = Subscription.SUBSCRIBE_NONE;
                    User user;
                    if (ttclient.GetUser(this.userid, out user))
                    {
                        if ((user.uLocalSubscriptions & Subscription.SUBSCRIBE_VIDEO) == Subscription.SUBSCRIBE_VIDEO)
                            subtype = Subscription.SUBSCRIBE_VIDEO;
                        else if ((user.uLocalSubscriptions & Subscription.SUBSCRIBE_INTERCEPT_VIDEO) == Subscription.SUBSCRIBE_INTERCEPT_VIDEO)
                            subtype = Subscription.SUBSCRIBE_INTERCEPT_VIDEO;
                    }

                    //store the command ID we get back from the client instance
                    unsubscribe_cmdid = ttclient.DoUnsubscribe(this.userid, subtype);
                    e.Cancel = true;
                }
                else
                    e.Cancel = true;
            }
            else
                ttclient.OnUserVideoFrame -= ttclient_OnUserVideoFrame;
        }

        void ttclient_OnCmdProcessing(int nCmdID, bool bComplete)
        {
            if (nCmdID == unsubscribe_cmdid && bComplete)
            {
                //now that the 'DoUnsubscribe' command has completed we can close the dialog.
                this.Close();
            }
        }

        void ttclient_OnUserVideoFrame(int nUserID, int nFrameQueueSize)
        {
            if (userid != nUserID)
                return;

            //Drain all images which are buffered, so we always get the latest.
            //Also ensure we use the same Bitmap so the 'ttclient' doesn't have to reallocate
            //a new bitmap every time 'GetUserVideoFrame' is called.
            while (nFrameQueueSize-- > 0)
            {
#if TT_ZEROCOPY
                if (bmp != null)
                    bmp.Dispose();
                bmp = null;

                //Release shared memory
                ttclient.ReleaseUserVideoFrame(nUserID);

                VideoFrame vidfrm = new VideoFrame();
                if (ttclient.AcquireUserVideoFrame(nUserID, ref vidfrm))
                {
                    //If we haven't received a key-frame in the current stream, 
                    //then skip display
                    if (stream_id == vidfrm.nStreamID || vidfrm.bKeyFrame)
                    {
                        //Create new bitmap from shared memory. 
                        //'vidfrm.frameBuffer' = Scan0 paramater which is shared
                        //memory. Never call 'ReleaseUserVideoFrame()' unless
                        //the Bitmap has already been disposed.
                        bmp = new Bitmap(vidfrm.nWidth, vidfrm.nHeight,
                                         4 * vidfrm.nWidth,
                                         PixelFormat.Format32bppRgb,
                                         vidfrm.frameBuffer);
                        stream_id = vidfrm.nStreamID;
                    }
                    else
                        Debug.WriteLine(String.Format("Skipped video frame from #{0} because we haven't received key-frame", userid));
                }
#else
                ttclient.GetUserVideoFrame(userid, ref bmp);
#endif
            }

            Invalidate();
        }

        protected override void OnPaintBackground(PaintEventArgs e)
        {
            if (bmp != null)
            {
                e.Graphics.DrawImage(bmp, 0, 0, this.ClientRectangle.Width,
                    this.ClientRectangle.Height);
            }
            else
            {
                base.OnPaintBackground(e);
            }
        }
        
        //hm, this doesn't work... :(
        //protected override void OnPaint(PaintEventArgs e)
        //{
        //    if (bmp != null)
        //    {
        //        ttclient.PaintVideoFrame(userid, e.Graphics.GetHdc(), 0, 0,  
        //            this.ClientRectangle.Width, this.ClientRectangle.Height);
        //        e.Graphics.ReleaseHdc();
        //    }
        //    else
        //        base.OnPaint(e);
        //}
    }
}